// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "assert/assert.hpp"

/**
 * Finds the next instance of a character within a string, safely (with a
 * limit). The returned pointer will be NULL if not found, or a pointer to a 
 * point within the string where the instance is.
 * 
 * @param haystack String to search.
 * @param needle Character to search for.
 * @param limit Max length you want to search for to limit yourself to.
 * @return Pointer to the character found, or NULL if not found.
 */
static inline char * stringFindNext(
  char *haystack,
  char needle,
  size_t limit
) {
  char *p;

  assertNotNull(haystack);
  assertTrue(limit > 0);

  for(p = haystack; (size_t)(p - haystack) < limit; p++) {
    if(*p == needle) return p;
    assertFalse(*p == '\0');// We don't allow you to have a limit > strlen
  }

  return NULL;
}

/**
 * Splits a string into a vector of strings, using a delimiter.
 * 
 * @param s String to split.
 * @param delim Delimiter to split by.
 * @return Vector of strings.
 */
static inline std::vector<std::string> stringSplit(
  const std::string &s,
  const std::string delim
) {
  size_t posStart = 0, posEnd, delimLength = delim.length();
  std::string token;
  std::vector<std::string> res;

  while((posEnd = s.find(delim, posStart)) != std::string::npos) {
    token = s.substr(posStart, posEnd - posStart);
    posStart = posEnd + delimLength;
    res.push_back (token);
  }

  res.push_back(s.substr(posStart));
  return res;
}

/**
 * Trims the whitespace from the left side of a string.
 * 
 * @param i Input string to trim.
 * @return Trimmed string.
 */
static inline std::string stringLTrim(const std::string &i) {
  std::string s = i;
  s.erase(s.begin(), std::find_if(s.begin(), s.end(), [](unsigned char ch) {
    return !std::isspace(ch);
  }));
  return s;
}

/**
 * Trims the whitespace from the right side of a string.
 * 
 * @param i Input string to trim.
 * @return Trimmed string.
 */
static inline std::string stringRTrim(const std::string &i) {
  std::string s = i;
  s.erase(std::find_if(s.rbegin(), s.rend(), [](unsigned char ch) {
    return !std::isspace(ch);
  }).base(), s.end());
  return s;
}

/**
 * Trims the whitespace from both sides of a string.
 * 
 * @param s Input string to trim.
 * @return Trimmed string.
 */
static inline std::string stringTrim(const std::string &s) {
  return stringLTrim(stringRTrim(s));
}